//I/O stuff
local configplayer
local changesoundsch

local function SaveCVar(cvarname, path)
	local file = io.openlocal(path, "w+")
	local cvar = CV_FindVar(cvarname)
	file:write(cvar.value)
	file:close()
end

local function LoadCVar(cvarname, path)
	local file = io.openlocal(path)
	local cvar = CV_FindVar(cvarname)
	if file
		local num = file:read("*n")
		CV_Set(cvar, num)
		file:close()
	end
end

local function LoadPVar(player, path, command, default)
	local file = io.openlocal(path)
    if file
		local val = file:read("*a")
		COM_BufInsertText(player, command.." "..val)
		file:close()
    else
        COM_BufInsertText(player, command.." "..default)
    end
end

local function SavePVar(player, path, toinsert)
	if io and player == consoleplayer
		local file = io.openlocal(path, "w+")
		file:write(tostring(toinsert))
		file:close()
	end
end

local function L_DecimalFixed(str)
	if str == nil return nil end
	local dec_offset = string.find(str,'%.')
	if dec_offset == nil
		return (tonumber(str) or 0)*FRACUNIT
	end
	
	local start, finish, _ = string.find(str,'%.%d+')
	local first
	if start == 1 then
		first = string.match(str, '%d+')
	else
		first = tonumber(string.match(str, '%d+')..string.match(str, '%d+', start))
	end
	local second = 10^(finish - start)
	
	return FixedDiv(first*FRACUNIT, second*FRACUNIT)
end

local function L_FixedDecimal(str,maxdecimal)
	if str == nil or tostring(str) == nil
		error("First argument is not a valid fixed value", 2)
		return
	end
	local number = tonumber(str)
	maxdecimal = ($ != nil) and $ or 3
	if tonumber(str) == 0 return '0' end
	local polarity = abs(number)/number
	local str_polarity = (polarity < 0) and '-' or ''
	local str_whole = tostring(abs(number/FRACUNIT))
	if maxdecimal == 0
		return str_polarity..str_whole
	end
	local decimal = number%FRACUNIT
	decimal = FRACUNIT + $
	decimal = FixedMul($,FRACUNIT*10^maxdecimal)
	decimal = $>>FRACBITS
	local str_decimal = string.sub(decimal,2)
	return str_polarity..str_whole..'.'..str_decimal
end

//Aura settings
CV_RegisterVar({
	name = "sfaura_sounds",
	defaultvalue = "On",
	flags = CV_SHOWMODIF|CV_CALL,
	PossibleValue = CV_OnOff,
	func = function(var)
		for player in players.iterate
			if var.value == 0
			and (player.auramo and player.auramo.valid) then
				S_StopSoundByID(player.auramo, SFA_SoundSchemes[player.aurasoundsch].passive)
			end
		end
	end
})

CV_RegisterVar({
	name = "sfaura_allowsounds",
	defaultvalue = "On",
	flags = CV_SHOWMODIF|CV_NETVAR|CV_CALL,
	PossibleValue = CV_OnOff,
	func = function(var)
		for player in players.iterate
			if var.value == 0
			and (player.auramo and player.auramo.valid) then
				S_StopSoundByID(player.auramo, SFA_SoundSchemes[player.aurasoundsch].passive)
				player.auramo.passfxtimer = 0
			end
		end
	end
})

local function SFA_XScale(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tostring(arg)
			if L_DecimalFixed(arg) > 0 and L_DecimalFixed(arg) <= 5*FRACUNIT then
				player.auraxscale = L_DecimalFixed(arg)
				SavePVar(player, "client/SuperFormAuras/sfaura_xscale.dat", arg)
				CONS_Printf(player, "Aura X size is now "..L_FixedDecimal(player.auraxscale))
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_xscale <value> (0-5, decimals allowed): Sets the aura's horizontal size (width).")
			CONS_Printf(player, "Current: "..L_FixedDecimal(player.auraxscale))
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_YScale(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tostring(arg)
			if L_DecimalFixed(arg) > 0 and L_DecimalFixed(arg) <= 5*FRACUNIT then
				player.aurayscale = L_DecimalFixed(arg)
				SavePVar(player, "client/SuperFormAuras/sfaura_yscale.dat", arg)
				CONS_Printf(player, "Aura Y size is now "..L_FixedDecimal(player.aurayscale))
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_yscale <value> (0-5, decimals allowed): Sets the aura's vertical size (height).")
			CONS_Printf(player, "Current: "..L_FixedDecimal(player.aurayscale))
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_XScale2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tostring(arg)
			if L_DecimalFixed(arg) > 0 and L_DecimalFixed(arg) <= 5*FRACUNIT then
				player.auraxscale2 = L_DecimalFixed(arg)
				SavePVar(player, "client/SuperFormAuras/sfaura_xscale2.dat", arg)
				CONS_Printf(player, "Second aura X size is now "..L_FixedDecimal(player.auraxscale2))
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_xscale2 <value> (0-5, decimals allowed): Sets the second aura's horizontal size (width).")
			CONS_Printf(player, "Current: "..L_FixedDecimal(player.auraxscale2))
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_YScale2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tostring(arg)
			if L_DecimalFixed(arg) > 0 and L_DecimalFixed(arg) <= 5*FRACUNIT then
				player.aurayscale2 = L_DecimalFixed(arg)
				SavePVar(player, "client/SuperFormAuras/sfaura_yscale2.dat", arg)
				CONS_Printf(player, "Second aura Y size is now "..L_FixedDecimal(player.aurayscale2))
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_yscale2 <value> (0-5, decimals allowed): Sets the second aura's vertical size (height).")
			CONS_Printf(player, "Current: "..L_FixedDecimal(player.aurayscale2))
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Conditions(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "never" then
				player.auraconditions = 0
				SavePVar(player, "client/SuperFormAuras/sfaura_conditions.dat", player.auraconditions)
				CONS_Printf(player, "Aura will now be always disabled.")
			elseif arg == "1" or arg == "super" then
				player.auraconditions = 1
				SavePVar(player, "client/SuperFormAuras/sfaura_conditions.dat", player.auraconditions)
				CONS_Printf(player, "Aura will now only trigger when you go Super.")
			elseif arg == "2" or arg == "always" then
				player.auraconditions = 2
				SavePVar(player, "client/SuperFormAuras/sfaura_conditions.dat", player.auraconditions)
				CONS_Printf(player, "Aura will now be always enabled.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_conditions <never / super / always>: Sets the conditions that must be met in order to trigger the aura.")
			if player.auraconditions == 0 then
				CONS_Printf(player, "Current: Never")
			elseif player.auraconditions == 1 then
				CONS_Printf(player, "Current: Super Only")
			else
				CONS_Printf(player, "Current: Always")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Color(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if R_GetColorByName(arg) != SKINCOLOR_NONE then
				player.auracolor = R_GetColorByName(arg)
				SavePVar(player, "client/SuperFormAuras/sfaura_color.dat", player.auracolor)
				CONS_Printf(player, "Aura color has been set to "..R_GetNameByColor(R_GetColorByName(arg)))
			elseif arg == "player" then
				player.auracolor = "player"
				SavePVar(player, "client/SuperFormAuras/sfaura_color.dat", player.auracolor)
				CONS_Printf(player, "Aura color will now be the same as the player's color.")
			elseif arg == "opposite" then
				player.auracolor = "opposite"
				SavePVar(player, "client/SuperFormAuras/sfaura_color.dat", player.auracolor)
				CONS_Printf(player, "Aura color will now be the opposite of the player's color.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_color <skincolor number/name>: Sets the aura's color.")
			if type(player.auracolor) == "number" then
				CONS_Printf(player, "Current aura color is "..R_GetNameByColor(player.auracolor))
			else
				CONS_Printf(player, "Current aura color is: "..player.auracolor)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Color2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if R_GetColorByName(arg) != SKINCOLOR_NONE then
				player.auracolor2 = R_GetColorByName(arg)
				SavePVar(player, "client/SuperFormAuras/sfaura_color2.dat", player.auracolor2)
				CONS_Printf(player, "Second aura color has been set to "..R_GetNameByColor(R_GetColorByName(arg)))
			elseif arg == "player" then
				player.auracolor2 = "player"
				SavePVar(player, "client/SuperFormAuras/sfaura_color2.dat", player.auracolor2)
				CONS_Printf(player, "Second aura color will now be the same as the player's color.")
			elseif arg == "opposite" then
				player.auracolor2 = "opposite"
				SavePVar(player, "client/SuperFormAuras/sfaura_color2.dat", player.auracolor2)
				CONS_Printf(player, "Second aura color will now be the opposite of the player's color.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_color2 <skincolor number/name>: Sets the second aura's color.")
			if type(player.auracolor2) == "number" then
				CONS_Printf(player, "Current second aura color is "..R_GetNameByColor(player.auracolor2))
			else
				CONS_Printf(player, "Current second aura color is: "..player.auracolor2)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_SoundScheme(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if SFA_SoundSchemes[arg] != nil
				if (player.auramo and player.auramo.valid) then
					S_StopSoundByID(player.auramo, SFA_SoundSchemes[player.aurasoundsch].passive)
					player.auramo.passfxtimer = 0
				end
				player.aurasoundsch = arg
				SavePVar(player, "client/SuperFormAuras/sfaura_soundscheme.dat", arg)
				if (player.auramo and player.auramo.valid) then
					S_StartSound(player.auramo, SFA_SoundSchemes[player.aurasoundsch].transform)
				end
				CONS_Printf(player, "Aura sound scheme is now "..SFA_SoundSchemes[arg].name)
			elseif arg == "list"
				for k, v in pairs(SFA_SoundSchemes)
					CONS_Printf(player, k.." - "..SFA_SoundSchemes[k].name)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_soundscheme <id>: Sets the sound scheme to be used by the aura.")
			CONS_Printf(player, "Current: "..SFA_SoundSchemes[player.aurasoundsch].name)
			CONS_Printf(player, "Type \"sfaura_soundscheme list\" to see all the available IDs for sound schemes.")
			
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Sprite(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if SFA_AuraStyles[arg] != nil
				if (player.auramo and player.auramo.valid) then
					player.auramo.frame = A|($ & ~FF_FRAMEMASK)
					player.auramo.nextframe = 0
					player.auramo.sprite = SFA_AuraStyles[arg].sprite
				end
				if (player.auraovmo and player.auraovmo.valid)
				and SFA_AuraStyles[arg].overlay then
					player.auraovmo.frame = A|($ & ~FF_FRAMEMASK)
					player.auraovmo.nextframe = 0
					player.auraovmo.sprite = SFA_AuraStyles[arg].overlay
				end
				player.aurastyle = arg
				SavePVar(player, "client/SuperFormAuras/sfaura_sprite.dat", arg)
				CONS_Printf(player, "Aura sprite style is now "..SFA_AuraStyles[arg].name)
			elseif arg == "list"
				for k, v in pairs(SFA_AuraStyles)
					CONS_Printf(player, k.." - "..SFA_AuraStyles[k].name)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_sprite <id>: Sets the sprite style of the aura.")
			CONS_Printf(player, "Current: "..SFA_AuraStyles[player.aurastyle].name)
			CONS_Printf(player, "Type \"sfaura_sprite list\" to see all the available IDs for aura styles.")
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Sprite2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if SFA_AuraStyles[arg] != nil
				if (player.auramo2 and player.auramo2.valid) then
					player.auramo2.frame = A|($ & ~FF_FRAMEMASK)
					player.auramo2.nextframe = 0
					player.auramo2.sprite = SFA_AuraStyles[arg].sprite
				end
				if (player.auraovmo2 and player.auraovmo2.valid)
				and SFA_AuraStyles[arg].overlay then
					player.auraovmo2.frame = A|($ & ~FF_FRAMEMASK)
					player.auraovmo2.nextframe = 0
					player.auraovmo2.sprite = SFA_AuraStyles[arg].overlay
				end
				player.aurastyle2 = arg
				SavePVar(player, "client/SuperFormAuras/sfaura_sprite2.dat", arg)
				CONS_Printf(player, "Second aura sprite style is now "..SFA_AuraStyles[arg].name)
			elseif arg == "list"
				for k, v in pairs(SFA_AuraStyles)
					CONS_Printf(player, k.." - "..SFA_AuraStyles[k].name)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_sprite2 <id>: Sets the sprite style of the second aura.")
			CONS_Printf(player, "Current: "..SFA_AuraStyles[player.aurastyle2].name)
			CONS_Printf(player, "Type \"sfaura_sprite2 list\" to see all the available IDs for aura styles.")
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Opacity(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tonumber(arg)
			if arg >= 0 and arg <= 10 then
				player.auraopacity = arg
				if player.auramo and player.auramo.valid then
					if player.auraopacity == 0 then
						player.auramo.flags2 = $|MF2_DONTDRAW
					elseif player.auraopacity > 0 and player.auraopacity < 10
						player.auramo.flags2 = $ & ~MF2_DONTDRAW
						player.auramo.frame = ($ & ~FF_TRANSMASK)|((10-player.auraopacity)<<FF_TRANSSHIFT)
					else
						player.auramo.frame = $ & ~FF_TRANSMASK
						player.auramo.flags2 = $ & ~MF2_DONTDRAW
					end
				end
				SavePVar(player, "client/SuperFormAuras/sfaura_opacity.dat", arg)
				if player.auraopacity > 0 then
					CONS_Printf(player, "Aura opacity is now "..player.auraopacity.."0%")
				else
					CONS_Printf(player, "Aura opacity is now "..player.auraopacity)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_opacity <value> (0-10): Sets the aura's opacity.")
			if player.auraopacity > 0 then
				CONS_Printf(player, "Current: "..player.auraopacity.."0%")
			else
				CONS_Printf(player, "Current: "..player.auraopacity)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Opacity2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tonumber(arg)
			if arg >= 0 and arg <= 10 then
				player.auraopacity2 = arg
				if player.auramo2 and player.auramo2.valid then
					if player.auraopacity2 == 0 then
						player.auramo2.flags2 = $|MF2_DONTDRAW
					elseif player.auraopacity2 > 0 and player.auraopacity2 < 10
						player.auramo2.flags2 = $ & ~MF2_DONTDRAW
						player.auramo2.frame = ($ & ~FF_TRANSMASK)|((10-player.auraopacity2)<<FF_TRANSSHIFT)
					else
						player.auramo2.frame = $ & ~FF_TRANSMASK
						player.auramo2.flags2 = $ & ~MF2_DONTDRAW
					end
				end
				SavePVar(player, "client/SuperFormAuras/sfaura_opacity2.dat", arg)
				if player.auraopacity2 > 0 then
					CONS_Printf(player, "Second aura opacity is now "..player.auraopacity2.."0%")
				else
					CONS_Printf(player, "Second aura opacity is now "..player.auraopacity2)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_opacity2 <value> (0-10): Sets the second aura's opacity.")
			if player.auraopacity > 0 then
				CONS_Printf(player, "Current: "..player.auraopacity2.."0%")
			else
				CONS_Printf(player, "Current: "..player.auraopacity2)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_BlendMode(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "none" or arg == "normal" then
				player.aurablend = AST_TRANSLUCENT
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have a normal blend mode.")
			elseif arg == "1" or arg == "add" then
				player.aurablend = AST_ADD
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have an additive blend mode.")
			elseif arg == "2" or arg == "subtract" then
				player.aurablend = AST_SUBTRACT
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have a subtractive blend mode.")
			elseif arg == "3" or arg == "difference" then
				player.aurablend = AST_REVERSESUBTRACT
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have a difference blend mode.")
			elseif arg == "4" or arg == "multiply" then
				player.aurablend = AST_MODULATE
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have a multiplicative blend mode.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_blendmode <normal / add / subtract / difference / multiply>: Sets the blending mode of the aura.")
			if player.aurablend == AST_TRANSLUCENT then
				CONS_Printf(player, "Current: Normal")
			elseif player.aurablend == AST_ADD then
				CONS_Printf(player, "Current: Additive")
			elseif player.aurablend == AST_SUBTRACT then
				CONS_Printf(player, "Current: Subtractive")
			elseif player.aurablend == AST_REVERSESUBTRACT then
				CONS_Printf(player, "Current: Difference")
			elseif player.aurablend == AST_MODULATE then
				CONS_Printf(player, "Current: Multiplicative")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_BlendMode2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "none" or arg == "normal" then
				player.aurablend2 = AST_TRANSLUCENT
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have a normal blend mode.")
			elseif arg == "1" or arg == "add" then
				player.aurablend2 = AST_ADD
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have an additive blend mode.")
			elseif arg == "2" or arg == "subtract" then
				player.aurablend2 = AST_SUBTRACT
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have a subtractive blend mode.")
			elseif arg == "3" or arg == "difference" then
				player.aurablend2 = AST_REVERSESUBTRACT
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have a difference blend mode.")
			elseif arg == "4" or arg == "multiply" then
				player.aurablend2 = AST_MODULATE
				SavePVar(player, "client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have a multiplicative blend mode.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_blendmode2 <normal / add / subtract / difference / multiply>: Sets the blending mode of the second aura.")
			if player.aurablend2 == AST_TRANSLUCENT then
				CONS_Printf(player, "Current: Normal")
			elseif player.aurablend2 == AST_ADD then
				CONS_Printf(player, "Current: Additive")
			elseif player.aurablend2 == AST_SUBTRACT then
				CONS_Printf(player, "Current: Subtractive")
			elseif player.aurablend2 == AST_REVERSESUBTRACT then
				CONS_Printf(player, "Current: Difference")
			elseif player.aurablend2 == AST_MODULATE then
				CONS_Printf(player, "Current: Multiplicative")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Outline(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "1" or arg == "on" then
				player.glowoutline = true
				SavePVar(player, "client/SuperFormAuras/sfaura_outline.dat", arg)
				CONS_Printf(player, "The player's sprites will now have a glowing outline.")
			elseif arg == "0" or arg == "off" then
				player.glowoutline = false
				SavePVar(player, "client/SuperFormAuras/sfaura_outline.dat", arg)
				CONS_Printf(player, "The player's sprites will no longer have a glowing outline.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_outline <on / off>: Sets if the player's sprites should have a glowing outline.")
			if player.glowoutline then
				CONS_Printf(player, "Current: Enabled")
			else
				CONS_Printf(player, "Current: Disabled")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_OutlineColor(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if R_GetColorByName(arg) != SKINCOLOR_NONE then
				player.outlinecolor = R_GetColorByName(arg)
				SavePVar(player, "client/SuperFormAuras/sfaura_outlinecolor.dat", player.outlinecolor)
				CONS_Printf(player, "Outline color has been set to "..R_GetNameByColor(R_GetColorByName(arg)))
			elseif arg == "player" then
				player.outlinecolor = "player"
				SavePVar(player, "client/SuperFormAuras/sfaura_outlinecolor.dat", player.outlinecolor)
				CONS_Printf(player, "Outline color will now be the same as the player's color.")
			elseif arg == "opposite" then
				player.outlinecolor = "opposite"
				SavePVar(player, "client/SuperFormAuras/sfaura_outlinecolor.dat", player.outlinecolor)
				CONS_Printf(player, "Outline color will now be the opposite of the player's color.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_outlinecolor <skincolor number/name>: Sets the color of the sprite's glowing outline.")
			if type(player.outlinecolor) == "number" then
				CONS_Printf(player, "Current outline color is "..R_GetNameByColor(player.outlinecolor))
			else
				CONS_Printf(player, "Current outline color is: "..player.outlinecolor)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_OutlineBlend(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "add" then
				player.outlineblend = AST_ADD
				SavePVar(player, "client/SuperFormAuras/sfaura_outlineblend.dat", arg)
				CONS_Printf(player, "The sprite outline will now have an additive blend mode.")
			elseif arg == "1" or arg == "subtract" then
				player.outlineblend = AST_SUBTRACT
				SavePVar(player, "client/SuperFormAuras/sfaura_outlineblend.dat", arg)
				CONS_Printf(player, "The sprite outline will now have a subtractive blend mode.")
			elseif arg == "2" or arg == "multiply" then
				player.outlineblend = AST_MODULATE
				SavePVar(player, "client/SuperFormAuras/sfaura_outlineblend.dat", arg)
				CONS_Printf(player, "The sprite outline will now have a multiplicative blend mode.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_outlineblend <add / subtract / multiply>: Sets the blending mode of the sprite outline.")
			if player.outlineblend == AST_ADD then
				CONS_Printf(player, "Current: Additive")
			elseif player.outlineblend == AST_SUBTRACT then
				CONS_Printf(player, "Current: Subtractive")
			elseif player.outlineblend == AST_MODULATE then
				CONS_Printf(player, "Current: Multiplicative")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

COM_AddCommand("sfaura_xscale", SFA_XScale)
COM_AddCommand("sfaura_yscale", SFA_YScale)
COM_AddCommand("sfaura_xscale2", SFA_XScale2)
COM_AddCommand("sfaura_yscale2", SFA_YScale2)
COM_AddCommand("sfaura_conditions", SFA_Conditions)
COM_AddCommand("sfaura_color", SFA_Color)
COM_AddCommand("sfaura_color2", SFA_Color2)
COM_AddCommand("sfaura_soundscheme", SFA_SoundScheme)
COM_AddCommand("sfaura_sprite", SFA_Sprite)
COM_AddCommand("sfaura_sprite2", SFA_Sprite2)
COM_AddCommand("sfaura_opacity", SFA_Opacity)
COM_AddCommand("sfaura_opacity2", SFA_Opacity2)
COM_AddCommand("sfaura_blendmode", SFA_BlendMode)
COM_AddCommand("sfaura_blendmode2", SFA_BlendMode2)
COM_AddCommand("sfaura_outline", SFA_Outline)
COM_AddCommand("sfaura_outlinecolor", SFA_OutlineColor)
COM_AddCommand("sfaura_outlineblend", SFA_OutlineBlend)

COM_AddCommand("sfaura_preset", function(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if SFA_Presets[arg] != nil then
				if SFA_Presets[arg].sfaura_xscale != nil then
					SFA_XScale(player, tostring(SFA_Presets[arg].sfaura_xscale))
				end
				if SFA_Presets[arg].sfaura_yscale != nil then
					SFA_YScale(player, tostring(SFA_Presets[arg].sfaura_yscale))
				end
				if SFA_Presets[arg].sfaura_xscale2 != nil then
					SFA_XScale2(player, tostring(SFA_Presets[arg].sfaura_xscale2))
				end
				if SFA_Presets[arg].sfaura_yscale2 != nil then
					SFA_YScale2(player, tostring(SFA_Presets[arg].sfaura_yscale2))
				end
				if SFA_Presets[arg].sfaura_opacity != nil then
					SFA_Opacity(player, tostring(SFA_Presets[arg].sfaura_opacity))
				end
				if SFA_Presets[arg].sfaura_opacity2 != nil then
					SFA_Opacity2(player, tostring(SFA_Presets[arg].sfaura_opacity2))
				end
				if SFA_Presets[arg].sfaura_color != nil then
					SFA_Color(player, tostring(SFA_Presets[arg].sfaura_color))
				end
				if SFA_Presets[arg].sfaura_color2 != nil then
					SFA_Color2(player, tostring(SFA_Presets[arg].sfaura_color2))
				end
				if SFA_Presets[arg].sfaura_sprite != nil then
					SFA_Sprite(player, tostring(SFA_Presets[arg].sfaura_sprite))
				end
				if SFA_Presets[arg].sfaura_sprite2 != nil then
					SFA_Sprite2(player, tostring(SFA_Presets[arg].sfaura_sprite2))
				end
				if SFA_Presets[arg].sfaura_soundscheme != nil then
					SFA_SoundScheme(player, tostring(SFA_Presets[arg].sfaura_soundscheme))
				end
				if SFA_Presets[arg].sfaura_blendmode != nil then
					SFA_BlendMode(player, tostring(SFA_Presets[arg].sfaura_blendmode))
				end
				if SFA_Presets[arg].sfaura_blendmode2 != nil then
					SFA_BlendMode2(player, tostring(SFA_Presets[arg].sfaura_blendmode2))
				end
				if SFA_Presets[arg].sfaura_outline != nil then
					SFA_Outline(player, tostring(SFA_Presets[arg].sfaura_outline))
				end
				if SFA_Presets[arg].sfaura_outlinecolor != nil then
					SFA_OutlineColor(player, tostring(SFA_Presets[arg].sfaura_outlinecolor))
				end
				if SFA_Presets[arg].sfaura_outlineblend != nil then
					SFA_OutlineBlend(player, tostring(SFA_Presets[arg].sfaura_outlineblend))
				end
				CONS_Printf(player, "Preset \""..SFA_Presets[arg].name.."\" loaded!")
			elseif arg == "list"
				for k, v in pairs(SFA_Presets)
					CONS_Printf(player, k.." - "..SFA_Presets[k].name)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_preset <id>: Changes all of your aura's settings from a preset.")
			CONS_Printf(player, "Type \"sfaura_preset list\" to see all the available IDs for presets.")
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end)

//I/O stuff
LoadCVar("sfaura_sounds", "client/SuperFormAuras/sfaura_sounds.dat")

addHook("GameQuit", function()
	SaveCVar("sfaura_sounds", "client/SuperFormAuras/sfaura_sounds.dat")
end)

local loaddelay = 3

addHook("ThinkFrame", do
	if not (consoleplayer and consoleplayer.valid)
	or not (consoleplayer.mo and consoleplayer.mo.valid) then return end

	if consoleplayer.sfaloadtimer == nil then
		consoleplayer.sfaloadtimer = 0
	end

	if not (configplayer and configplayer.valid) then
		consoleplayer.sfaloadtimer = $+1
	
		if consoleplayer.sfaloadtimer == loaddelay then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_color.dat", "sfaura_color", "player")
		elseif consoleplayer.sfaloadtimer == loaddelay*2 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_color2.dat", "sfaura_color2", "player")
		elseif consoleplayer.sfaloadtimer == loaddelay*3 then	
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_soundscheme.dat", "sfaura_soundscheme", "ssj")
		elseif consoleplayer.sfaloadtimer == loaddelay*4 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_sprite.dat", "sfaura_sprite", "ssj")
		elseif consoleplayer.sfaloadtimer == loaddelay*5 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_sprite2.dat", "sfaura_sprite2", "ssj")
		elseif consoleplayer.sfaloadtimer == loaddelay*6 then	
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_opacity.dat", "sfaura_opacity", 10)
		elseif consoleplayer.sfaloadtimer == loaddelay*7 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_opacity2.dat", "sfaura_opacity2", 0)
		elseif consoleplayer.sfaloadtimer == loaddelay*8 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_xscale.dat", "sfaura_xscale", 1)
		elseif consoleplayer.sfaloadtimer == loaddelay*9 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_xscale2.dat", "sfaura_xscale2", 1)
		elseif consoleplayer.sfaloadtimer == loaddelay*10 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_yscale.dat", "sfaura_yscale", 1)
		elseif consoleplayer.sfaloadtimer == loaddelay*11 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_yscale2.dat", "sfaura_yscale2", 1)
		elseif consoleplayer.sfaloadtimer == loaddelay*12 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_blendmode.dat", "sfaura_blendmode", 0)
		elseif consoleplayer.sfaloadtimer == loaddelay*13 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_blendmode2.dat", "sfaura_blendmode2", 0)
		elseif consoleplayer.sfaloadtimer == loaddelay*14 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_outline.dat", "sfaura_outline", 0)
		elseif consoleplayer.sfaloadtimer == loaddelay*15 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_outlinecolor.dat", "sfaura_outlinecolor", 2)
		elseif consoleplayer.sfaloadtimer == loaddelay*16 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_outlineblend.dat", "sfaura_outlineblend", 0)
		elseif consoleplayer.sfaloadtimer == loaddelay*17 then
			LoadPVar(consoleplayer, "client/SuperFormAuras/sfaura_conditions.dat", "sfaura_conditions", 1)
			consoleplayer.sfaloadtimer = 0
			configplayer = consoleplayer
		end
	end
end)